@@ -602,6 +602,7 @@ struct HTMLDocumentObj {
     DOCHOSTUIINFO hostinfo;
 
     IOleUndoManager *undomgr;
+    IHTMLEditServices *editsvcs;
 
     HWND hwnd;
     HWND tooltips_hwnd;
@@ -775,6 +776,8 @@ struct HTMLDocumentNode {
 
     UINT charset;
 
+    unsigned unique_id;
+
     struct list selection_list;
     struct list range_list;
     struct list plugin_hosts;
