@@ -20,11 +20,13 @@
 
 #include <wine/test.h>
 #include <stdarg.h>
+#include <stdio.h>
 
 #include "windef.h"
 #include "winbase.h"
 #include "ole2.h"
 #include "mshtml.h"
+#include "objsafe.h"
 
 static BSTR a2bstr(const char *str)
 {
@@ -72,6 +74,8 @@ static int strcmp_wa(LPCWSTR strw, const char *stra)
 
 static IHTMLXMLHttpRequest *xhr = NULL;
 static BSTR content_type = NULL;
+static int loading_cnt = 0;
+static int readystatechange_cnt = 0;
 
 DEFINE_EXPECT(xmlhttprequest_onreadystatechange_opened);
 DEFINE_EXPECT(xmlhttprequest_onreadystatechange_headers_received);
@@ -241,6 +245,7 @@ static HRESULT WINAPI xmlhttprequest_onreadystatechange(IDispatchEx *iface, DISP
 
     hres = IHTMLXMLHttpRequest_get_readyState(xhr, &val);
     ok(hres == S_OK, "get_readyState failed: %08x\n", hres);
+    readystatechange_cnt++;
 
     switch(val) {
         case 1:
@@ -250,7 +255,8 @@ static HRESULT WINAPI xmlhttprequest_onreadystatechange(IDispatchEx *iface, DISP
             CHECK_EXPECT(xmlhttprequest_onreadystatechange_headers_received);
             break;
         case 3:
-            CHECK_EXPECT(xmlhttprequest_onreadystatechange_loading);
+            loading_cnt++;
+            CHECK_EXPECT2(xmlhttprequest_onreadystatechange_loading);
             break;
         case 4:
             CHECK_EXPECT(xmlhttprequest_onreadystatechange_done);
@@ -375,7 +381,6 @@ static void pump_msgs(BOOL *b)
     }
 }
 
-static const char EXPECT_RESPONSE_TEXT[] = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<a>TEST</a>\n";
 
 struct HEADER_TYPE {
     const char *key;
@@ -451,7 +456,110 @@ static void test_header(const struct HEADER_TYPE expect[], int num)
     }
 }
 
-static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
+static const char *debugstr_variant(const VARIANT *var)
+{
+    static char buf[400];
+
+    if (!var)
+        return "(null)";
+
+    switch (V_VT(var))
+    {
+    case VT_EMPTY:
+        return "{VT_EMPTY}";
+    case VT_BSTR:
+        sprintf(buf, "{VT_BSTR: %s}", wine_dbgstr_w(V_BSTR(var)));
+        break;
+    case VT_BOOL:
+        sprintf(buf, "{VT_BOOL: %x}", V_BOOL(var));
+        break;
+    case VT_UI4:
+        sprintf(buf, "{VT_UI4: %u}", V_UI4(var));
+        break;
+    default:
+        sprintf(buf, "{vt %d}", V_VT(var));
+        break;
+    }
+
+    return buf;
+}
+
+static void test_illegal_xml(IXMLDOMDocument *xmldom)
+{
+    IXMLDOMNode *first, *last;
+    VARIANT variant;
+    HRESULT hres;
+    BSTR bstr;
+
+    hres = IXMLDOMDocument_get_baseName(xmldom, NULL);
+    ok(hres == E_INVALIDARG, "Expect E_INVALIDARG, got %08x\n", hres);
+    hres = IXMLDOMDocument_get_baseName(xmldom, &bstr);
+    ok(hres == S_FALSE, "get_baseName failed: %08x\n", hres);
+    ok(bstr == NULL, "bstr(%p): %s\n", bstr, wine_dbgstr_w(bstr));
+    SysFreeString(bstr);
+
+    hres = IXMLDOMDocument_get_dataType(xmldom, NULL);
+    ok(hres == E_INVALIDARG, "Expect E_INVALIDARG, got %08x\n", hres);
+    hres = IXMLDOMDocument_get_dataType(xmldom, &variant);
+    ok(hres == S_FALSE, "get_dataType failed: %08x\n", hres);
+    ok(V_VT(&variant) == VT_NULL, "got %s\n", debugstr_variant(&variant));
+    VariantClear(&variant);
+
+    hres = IXMLDOMDocument_get_text(xmldom, &bstr);
+    ok(!strcmp_wa(bstr, ""), "text = %s\n", wine_dbgstr_w(bstr));
+    SysFreeString(bstr);
+
+    hres = IXMLDOMDocument_get_firstChild(xmldom, NULL);
+    ok(hres == E_INVALIDARG, "Expect E_INVALIDARG, got %08x\n", hres);
+
+    first = (void*)0xdeadbeef;
+    hres = IXMLDOMDocument_get_firstChild(xmldom, &first);
+    ok(hres == S_FALSE, "get_firstChild failed: %08x\n", hres);
+    ok(first == NULL, "first != NULL\n");
+
+    last = (void*)0xdeadbeef;
+    hres = IXMLDOMDocument_get_lastChild(xmldom, &last);
+    ok(hres == S_FALSE, "get_lastChild failed: %08x\n", hres);
+    ok(last == NULL, "last != NULL\n");
+}
+
+static void test_responseXML(const char *expect_text)
+{
+    IDispatch *disp;
+    IXMLDOMDocument *xmldom;
+    IObjectSafety *safety;
+    DWORD enabled = 0, supported = 0;
+    HRESULT hres;
+
+    disp = NULL;
+    hres = IHTMLXMLHttpRequest_get_responseXML(xhr, &disp);
+    ok(hres == S_OK, "get_responseXML failed: %08x\n", hres);
+    ok(disp != NULL, "disp == NULL\n");
+
+    xmldom = NULL;
+    hres = IDispatch_QueryInterface(disp, &IID_IXMLDOMDocument, (void**)&xmldom);
+    ok(hres == S_OK, "QueryInterface(IXMLDOMDocument) failed: %08x\n", hres);
+    ok(xmldom != NULL, "xmldom == NULL\n");
+
+    hres = IXMLDOMDocument_QueryInterface(xmldom, &IID_IObjectSafety, (void**)&safety);
+    ok(hres == S_OK, "QueryInterface IObjectSafety failed: %08x\n", hres);
+    hres = IObjectSafety_GetInterfaceSafetyOptions(safety, NULL, &supported, &enabled);
+    ok(hres == S_OK, "GetInterfaceSafetyOptions failed: %08x\n", hres);
+    ok(broken(supported == (INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA)) ||
+       supported == (INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA | INTERFACE_USES_SECURITY_MANAGER) /* msxml3 SP8+ */,
+        "Expected supported: (INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA | INTERFACE_USES_SECURITY_MANAGER), got %08x\n", supported);
+    ok(enabled == (INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA | INTERFACE_USES_SECURITY_MANAGER),
+        "Expected enabled: (INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA | INTERFACE_USES_SECURITY_MANAGER), got 0x%08x\n", enabled);
+    IObjectSafety_Release(safety);
+
+    if(!expect_text)
+        test_illegal_xml(xmldom);
+
+    IXMLDOMDocument_Release(xmldom);
+    IDispatch_Release(disp);
+}
+
+static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url, const char *expect_text)
 {
     VARIANT vbool, vempty, var;
     BSTR method, url;
@@ -465,6 +573,8 @@ static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
         {"Content-Type", "application/xml"}
     };
 
+    trace("test_sync_xhr\n");
+
     create_xmlhttprequest(doc);
     if(!xhr)
         return;
@@ -566,12 +676,14 @@ static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
     SET_EXPECT(xmlhttprequest_onreadystatechange_headers_received);
     SET_EXPECT(xmlhttprequest_onreadystatechange_loading);
     SET_EXPECT(xmlhttprequest_onreadystatechange_done);
+    loading_cnt = 0;
     hres = IHTMLXMLHttpRequest_send(xhr, vempty);
     ok(hres == S_OK, "send failed: %08x\n", hres);
     CHECK_CALLED(xmlhttprequest_onreadystatechange_opened);
     CHECK_CALLED(xmlhttprequest_onreadystatechange_headers_received);
     CHECK_CALLED(xmlhttprequest_onreadystatechange_loading);
     CHECK_CALLED(xmlhttprequest_onreadystatechange_done);
+    ok(loading_cnt == 1, "loading_cnt = %d\n", loading_cnt);
 
     text = NULL;
     hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, content_type, &text);
@@ -579,7 +691,8 @@ static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
     ok(text != NULL, "text == NULL\n");
     SysFreeString(text);
 
-    test_header(expect_headers, sizeof(expect_headers)/sizeof(expect_headers[0]));
+    if(expect_text)
+        test_header(expect_headers, sizeof(expect_headers)/sizeof(expect_headers[0]));
 
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
@@ -601,15 +714,18 @@ static void test_sync_xhr(IHTMLDocument2 *doc, const char *xml_url)
     hres = IHTMLXMLHttpRequest_get_responseText(xhr, &text);
     ok(hres == S_OK, "get_responseText failed: %08x\n", hres);
     ok(text != NULL, "test == NULL\n");
-    ok(!strcmp_wa(text, EXPECT_RESPONSE_TEXT), "expect %s, got %s\n",
-            EXPECT_RESPONSE_TEXT, wine_dbgstr_w(text));
+    if(expect_text)
+        ok(!strcmp_wa(text, expect_text), "expect %s, got %s\n",
+            expect_text, wine_dbgstr_w(text));
     SysFreeString(text);
 
+    test_responseXML(expect_text);
+
     IHTMLXMLHttpRequest_Release(xhr);
     xhr = NULL;
 }
 
-static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
+static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url, const char *expect_text)
 {
     VARIANT vbool, vempty, var;
     BSTR method, url;
@@ -722,6 +838,7 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
     SET_EXPECT(xmlhttprequest_onreadystatechange_headers_received);
     SET_EXPECT(xmlhttprequest_onreadystatechange_loading);
     SET_EXPECT(xmlhttprequest_onreadystatechange_done);
+    loading_cnt = 0;
     hres = IHTMLXMLHttpRequest_send(xhr, vempty);
 
     ok(hres == S_OK, "send failed: %08x\n", hres);
@@ -731,6 +848,11 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
     CHECK_CALLED(xmlhttprequest_onreadystatechange_headers_received);
     CHECK_CALLED(xmlhttprequest_onreadystatechange_loading);
     CHECK_CALLED(xmlhttprequest_onreadystatechange_done);
+    /* Workaround for loading large files */
+    if(expect_text)
+        ok(loading_cnt == 1, "loading_cnt = %d\n", loading_cnt);
+    else
+        todo_wine ok(loading_cnt == 1, "loading_cnt = %d\n", loading_cnt);
 
     if(FAILED(hres)) {
         IHTMLXMLHttpRequest_Release(xhr);
@@ -744,14 +866,8 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
     ok(text != NULL, "text == NULL\n");
     SysFreeString(text);
 
-    text = NULL;
-    hres = IHTMLXMLHttpRequest_getResponseHeader(xhr, content_type, &text);
-    ok(hres == S_OK, "getResponseHeader failed, got %08x\n", hres);
-    ok(text != NULL, "text == NULL\n");
-    ok(!strcmp_wa(text, "application/xml"), "text = %s\n", wine_dbgstr_w(text));
-    SysFreeString(text);
-
-    test_header(expect_headers, sizeof(expect_headers)/sizeof(expect_headers[0]));
+    if(expect_text)
+        test_header(expect_headers, sizeof(expect_headers)/sizeof(expect_headers[0]));
 
     val = 0xdeadbeef;
     hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
@@ -774,14 +890,99 @@ static void test_async_xhr(IHTMLDocument2 *doc, const char *xml_url)
     hres = IHTMLXMLHttpRequest_get_responseText(xhr, &text);
     ok(hres == S_OK, "get_responseText failed: %08x\n", hres);
     ok(text != NULL, "test == NULL\n");
-    ok(!strcmp_wa(text, EXPECT_RESPONSE_TEXT), "expect %s, got %s\n",
-        EXPECT_RESPONSE_TEXT, wine_dbgstr_w(text));
+    if(expect_text)
+        ok(!strcmp_wa(text, expect_text), "expect %s, got %s\n",
+            expect_text, wine_dbgstr_w(text));
     SysFreeString(text);
 
+    test_responseXML(expect_text);
+
     IHTMLXMLHttpRequest_Release(xhr);
     xhr = NULL;
 }
 
+static void test_async_xhr_abort(IHTMLDocument2 *doc, const char *xml_url)
+{
+    VARIANT vbool, vempty, var;
+    BSTR method, url;
+    LONG val;
+    HRESULT hres;
+
+    method = a2bstr("GET");
+    url = a2bstr(xml_url);
+    V_VT(&vbool) = VT_BOOL;
+    V_BOOL(&vbool) = VARIANT_TRUE;
+    V_VT(&vempty) = VT_EMPTY;
+
+    trace("abort before send() is fired\n");
+    create_xmlhttprequest(doc);
+    if(!xhr)
+        return;
+
+    V_VT(&var) = VT_DISPATCH;
+    V_DISPATCH(&var) = (IDispatch*)&xmlhttprequest_onreadystatechange_obj;
+    hres = IHTMLXMLHttpRequest_put_onreadystatechange(xhr, var);
+
+    SET_EXPECT(xmlhttprequest_onreadystatechange_opened);
+    hres = IHTMLXMLHttpRequest_open(xhr, method, url, vbool, vempty, vempty);
+    ok(hres == S_OK, "open failed: %08x\n", hres);
+    CHECK_CALLED(xmlhttprequest_onreadystatechange_opened);
+
+    hres = IHTMLXMLHttpRequest_abort(xhr);
+    ok(hres == S_OK, "abort failed: %08x\n", hres);
+
+    hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
+    ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
+    ok(val == 0, "Expect 0, got %d\n", val);
+
+    hres = IHTMLXMLHttpRequest_get_readyState(xhr, &val);
+    ok(hres == S_OK, "get_readyState failed: %08x\n", hres);
+    ok(val == 0, "Expect UNSENT, got %d\n", val);
+
+    IHTMLXMLHttpRequest_Release(xhr);
+    xhr = NULL;
+
+    trace("abort after send() is fired\n");
+    create_xmlhttprequest(doc);
+    V_VT(&var) = VT_DISPATCH;
+    V_DISPATCH(&var) = (IDispatch*)&xmlhttprequest_onreadystatechange_obj;
+    hres = IHTMLXMLHttpRequest_put_onreadystatechange(xhr, var);
+
+    SET_EXPECT(xmlhttprequest_onreadystatechange_opened);
+    hres = IHTMLXMLHttpRequest_open(xhr, method, url, vbool, vempty, vempty);
+    ok(hres == S_OK, "open failed: %08x\n", hres);
+    CHECK_CALLED(xmlhttprequest_onreadystatechange_opened);
+
+    loading_cnt = 0;
+    readystatechange_cnt = 0;
+    SET_EXPECT(xmlhttprequest_onreadystatechange_opened);
+    SET_EXPECT(xmlhttprequest_onreadystatechange_done);
+    hres = IHTMLXMLHttpRequest_send(xhr, vempty);
+    ok(hres == S_OK, "send failed: %08x\n", hres);
+    todo_wine CHECK_CALLED(xmlhttprequest_onreadystatechange_opened);
+
+    hres = IHTMLXMLHttpRequest_abort(xhr);
+    ok(hres == S_OK, "abort failed: %08x\n", hres);
+    CHECK_CALLED(xmlhttprequest_onreadystatechange_done);
+
+    hres = IHTMLXMLHttpRequest_get_readyState(xhr, &val);
+    ok(hres == S_OK, "get_readyState failed: %08x\n", hres);
+    ok(val == 0, "Expect UNSENT, got %d\n", val);
+
+    hres = IHTMLXMLHttpRequest_get_status(xhr, &val);
+    ok(hres == E_FAIL, "Expect E_FAIL, got: %08x\n", hres);
+    ok(val == 0, "Expect 0, got %d\n", val);
+
+    ok(loading_cnt == 0, "loading_cnt = %d, expect 0, loading_cnt\n", loading_cnt);
+    todo_wine ok(readystatechange_cnt == 2, "readystatechange_cnt = %d, expect 2\n", readystatechange_cnt);
+
+    IHTMLXMLHttpRequest_Release(xhr);
+    xhr = NULL;
+
+    SysFreeString(method);
+    SysFreeString(url);
+}
+
 static IHTMLDocument2 *create_doc_from_url(const char *start_url)
 {
     BSTR url;
@@ -829,14 +1030,19 @@ START_TEST(xmlhttprequest)
     IHTMLDocument2 *doc;
     static const char start_url[] = "http://test.winehq.org/tests/hello.html";
     static const char xml_url[] = "http://test.winehq.org/tests/xmltest.xml";
+    static const char large_page_url[] = "http://test.winehq.org/tests/data.php";
+    static const char expect_response_text[] = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<a>TEST</a>\n";
 
     CoInitialize(NULL);
 
     content_type = a2bstr("Content-Type");
     doc = create_doc_from_url(start_url);
     if(doc) {
-        test_sync_xhr(doc, xml_url);
-        test_async_xhr(doc, xml_url);
+        test_sync_xhr(doc, xml_url, expect_response_text);
+        test_sync_xhr(doc, large_page_url, NULL);
+        test_async_xhr(doc, xml_url, expect_response_text);
+        test_async_xhr(doc, large_page_url, NULL);
+        test_async_xhr_abort(doc, large_page_url);
         IHTMLDocument2_Release(doc);
     }
     SysFreeString(content_type);
