@@ -325,6 +325,15 @@ int CDECL MSVCRT_atexit(void (*func)(void))
   return MSVCRT__onexit((MSVCRT__onexit_t)func) == (MSVCRT__onexit_t)func ? 0 : -1;
 }
 
+/*********************************************************************
+ *		_crt_atexit (UCRTBASE.@)
+ */
+int CDECL MSVCRT__crt_atexit(void (*func)(void))
+{
+  TRACE("(%p)\n", func);
+  return MSVCRT__onexit((MSVCRT__onexit_t)func) == (MSVCRT__onexit_t)func ? 0 : -1;
+}
+
 /*********************************************************************
  *		_set_purecall_handler (MSVCR71.@)
  */
