@@ -36,6 +36,7 @@
 #include "ddk/csq.h"
 #include "ddk/ntddk.h"
 #include "ddk/ntifs.h"
+#include "ddk/wdm.h"
 #include "wine/unicode.h"
 #include "wine/server.h"
 #include "wine/list.h"
@@ -1430,6 +1431,24 @@ NTSTATUS WINAPI ExCreateCallback(PCALLBACK_OBJECT *obj, POBJECT_ATTRIBUTES attr,
 }
 
 
+/***********************************************************************
+ *           ExDeleteNPagedLookasideList   (NTOSKRNL.EXE.@)
+ */
+void WINAPI ExDeleteNPagedLookasideList( PNPAGED_LOOKASIDE_LIST lookaside )
+{
+    FIXME("(%p) stub\n", lookaside);
+}
+
+
+/***********************************************************************
+ *           ExDeletePagedLookasideList  (NTOSKRNL.EXE.@)
+ */
+void WINAPI ExDeletePagedLookasideList( PPAGED_LOOKASIDE_LIST lookaside )
+{
+    FIXME("(%p) stub\n", lookaside);
+}
+
+
 /***********************************************************************
  *           ExFreePool   (NTOSKRNL.EXE.@)
  */
@@ -2047,6 +2066,16 @@ NTSTATUS WINAPI PsSetCreateThreadNotifyRoutine( PCREATE_THREAD_NOTIFY_ROUTINE No
 }
 
 
+/***********************************************************************
+ *           PsRemoveCreateThreadNotifyRoutine  (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI PsRemoveCreateThreadNotifyRoutine( PCREATE_THREAD_NOTIFY_ROUTINE NotifyRoutine )
+{
+    FIXME( "stub: %p\n", NotifyRoutine );
+    return STATUS_SUCCESS;
+}
+
+
 /***********************************************************************
  *           PsTerminateSystemThread   (NTOSKRNL.EXE.@)
  */
@@ -2280,6 +2309,24 @@ NTSTATUS WINAPI IoCsqInitialize(PIO_CSQ csq, PIO_CSQ_INSERT_IRP insert_irp, PIO_
     return STATUS_SUCCESS;
 }
 
+/***********************************************************************
+ *           ExAcquireResourceExclusiveLite  (NTOSKRNL.EXE.@)
+ */
+BOOLEAN WINAPI ExAcquireResourceExclusiveLite( PERESOURCE resource, BOOLEAN wait )
+{
+    FIXME( ":%p %u stub\n", resource, wait );
+    return TRUE;
+}
+
+/***********************************************************************
+ *           ExDeleteResourceLite  (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI ExDeleteResourceLite(PERESOURCE resource)
+{
+    FIXME("(%p): stub\n", resource);
+    return STATUS_NOT_IMPLEMENTED;
+}
+
 /*****************************************************
  *           ExInterlockedRemoveHeadList  (NTOSKRNL.EXE.@)
  */
@@ -2303,6 +2350,14 @@ PLIST_ENTRY WINAPI ExfInterlockedRemoveHeadList(PLIST_ENTRY head, PKSPIN_LOCK lo
     return ExInterlockedRemoveHeadList( head, lock );
 }
 
+/***********************************************************************
+ *           ExReleaseResourceForThreadLite   (NTOSKRNL.EXE.@)
+ */
+void WINAPI ExReleaseResourceForThreadLite( PERESOURCE resource, ERESOURCE_THREAD tid )
+{
+    FIXME( "stub: %p %lu\n", resource, tid );
+}
+
 /***********************************************************************
  *           KeEnterCriticalRegion  (NTOSKRNL.EXE.@)
  */
@@ -2318,3 +2373,37 @@ void WINAPI KeLeaveCriticalRegion(void)
 {
     FIXME(": stub\n");
 }
+
+/***********************************************************************
+ *           ProbeForRead   (NTOSKRNL.EXE.@)
+ */
+void WINAPI ProbeForRead(void *address, SIZE_T length, ULONG alignment)
+{
+    FIXME("(%p %lu %u) stub\n", address, length, alignment);
+}
+
+/***********************************************************************
+ *           ProbeForWrite   (NTOSKRNL.EXE.@)
+ */
+void WINAPI ProbeForWrite(void *address, SIZE_T length, ULONG alignment)
+{
+    FIXME("(%p %lu %u) stub\n", address, length, alignment);
+}
+
+/***********************************************************************
+ *           CmRegisterCallback  (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI CmRegisterCallback(EX_CALLBACK_FUNCTION *function, void *context, LARGE_INTEGER *cookie)
+{
+    FIXME("(%p %p %p): stub\n", function, context, cookie);
+    return STATUS_NOT_IMPLEMENTED;
+}
+
+/***********************************************************************
+ *           CmUnRegisterCallback  (NTOSKRNL.EXE.@)
+ */
+NTSTATUS WINAPI CmUnRegisterCallback(LARGE_INTEGER cookie)
+{
+    FIXME("(%s): stub\n", wine_dbgstr_longlong(cookie.QuadPart));
+    return STATUS_NOT_IMPLEMENTED;
+}
